#
# Copyright (c) 2002 Gilles Lenfant
# All rights reserved. Written by Gilles Lenfant <glenfant@bigfoot.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
'''
This is the XQuizz product for Zope

An XQuizz object provide basic features to include vote, poll, quizz (...)
in a Zope site.

The content of an XQuizz object is provided by an XML text that lists
the questions and their possible answers.

The ZQuizz provides :

* HTML form rendering,
* assessment of user replies.

$Id: XQuizz.py,v 1.1.1.1 2002/05/27 02:22:35 glt Exp $
'''

__version__ = "$Revision: 1.1.1.1 $"[11:-2]

import string

from Globals import DTMLFile, MessageDialog
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens

from xquizzparser import parseXQuizz

manage_addXQuizzForm = DTMLFile('dtml/addXQuizz_form', globals())

def manage_addXQuizz(self, id, REQUEST=None):
    '''Create/record the new XQuizz
    '''
    zq = XQuizz(id, title='')
    self._setObject(id, zq)
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+('/%s/manage_main' % id))
    return

class XQuizz(SimpleItem):
    '''Provides poll like features.
    More info in the README tab of /Control_Panel/Products/XQuizz product
    '''
    meta_type = 'XQuizz'

    manage_options = ({'label': 'Edit', 'action': 'manage_main'},
                      {'label': 'Test', 'action': 'manage_testForm'}
                      ) + SimpleItem.manage_options


    security = ClassSecurityInfo()

    security.declarePublic('quizz')
    def __init__(self, id, title=''):
        self.id = id
        self.title = title
        self.xmlquizz = SampleXmlQuizz
        self.encoding = 'ISO-8859-1'
        dummmy, self.quizz = parseXQuizz(SampleXmlQuizz)
        return

    manage_main = DTMLFile('dtml/editForm', globals())

    security.declareProtected(view_management_screens, 'manage_edit')
    def manage_edit(self, title='', xmlquizz='', encoding='', REQUEST=None):
        '''Changes the properties of a XQuizz
        '''
        if REQUEST is not None:
            title = string.strip(REQUEST.form['title'])
            xmlquizz = string.strip(REQUEST.form['xmlquizz'])
            encoding = string.strip(REQUEST.form['encoding'])
        errors, quizz = parseXQuizz(xmlquizz, encoding)
        self.title = title
        self.xmlquizz = xmlquizz
        self.encoding = encoding
        self.quizz = quizz
        if REQUEST is not None:
            if errors:
                return self.manage_main(self,REQUEST, manage_tabs_message='Content changed.', errors=errors)
            else:
                return self.manage_main(self,REQUEST, manage_tabs_message='Content changed.')
        return errors

    manage_testForm = DTMLFile('dtml/testForm', globals())

    security.declareProtected(view_management_screens, 'manage_test')
    def manage_test(self, REQUEST=None):
        '''Shows the result of the test form
        '''
        maxi = self.quizz.maximumScore()
        score = self.quizz.getScore(REQUEST.form)
        message = ('Your score is <b>%d</b> points.<br>'
                   'The bets possible score is <b>%d</b> points.<br><br>'
                   'Best answers are:<br><br>%s')
        best = ''
        iq = 0
        for question in self.quizz:
            iq = iq + 1
            best = best + '- Question %d: "%s"<br>' % (iq, question.bestAnswer())
        return MessageDialog(title="OK", message=message % (score, maxi, best), action='manage_testForm')

    security.declarePublic('render')
    render = DTMLFile('dtml/render', globals())
# /class XQuizz

def initialize(context):
    context.registerClass(
        instance_class = XQuizz,
        permission='Add XQuizz',
        constructors=(manage_addXQuizzForm, manage_addXQuizz),
        icon='www/XQuizz_icon.gif')
    return

SampleXmlQuizz = '''<?xml version="1.0" encoding="ISO-8859-1"?>
<!--
You will find the commented DTD for this document's format in the README tab of the product.
Warning... As this is XML, use:
&lt; to render <
&gt; to render >
&amp to render &
-->

<quizz>
    <question>
        <label>Do you think this product is useful?</label>
        <answer value="0">No</answer>
        <answer value="1">Maybe</answer>
        <answer value="2">Yes</answer>
    </question>
    <question>
        <!-- Quote HTML content like in the example below -->
        <label><![CDATA[Do you think the <b>author</b> deserves more money?]]></label>
        <answer value="0">No</answer>
        <answer value="1">Yes</answer>
    </question>
</quizz>'''