#!/usr/bin/env python2.1

####################################
## Testing features from quizz.py ##
####################################
import sys
sys.path.append('..')
import unittest
from quizz import Quizz, Question, Answer

class TestCaseBase(unittest.TestCase):
    def setUp(self):
        self.a0_0 = Answer(label='answer 0_0', points=0)
        self.a0_1 = Answer(label='answer 0_1', points=1)
        self.a0_2 = Answer(label='answer 0_2', points=2)
        self.a1_0 = Answer(label='answer 1_0', points=0)
        self.a1_1 = Answer(label='answer 1_1', points=1)
        self.a1_2 = Answer(label='answer 1_2', points=2)
        self.q0 = Question(label='question 0', initanswers=[self.a0_0, self.a0_1, self.a0_2])
        self.q1 = Question(label='question 0', initanswers=[self.a1_0, self.a1_1, self.a1_2])
        return
    
    def tearDown(self):
        pass

####################
## Class Question ##
####################

class TestQuestion(TestCaseBase):
    def testCreate1(self):
        q0 = Question(label='question_0', initanswers=[self.a0_0, self.a0_1])
        self.failUnless(len(q0) == 2, 'Failed')
        return
    
    def testCreate2(self):
        q0 = Question(label='question_0', initanswers=[self.a0_0, self.a0_1])
        q0.addAnswer(self.a0_2)
        self.failUnless(len(q0) == 3, 'Failed')
        return

    def testMaxiscore(self):
        q0 = Question(label='question_0', initanswers=[self.a0_0, self.a0_1, self.a0_2])
        self.failUnless(q0.maximumScore() == 2, 'Failed')
        return

    def testAnswerId(self):
        q0 = Question(label='question_0', initanswers=[self.a0_0, self.a0_1, self.a0_2])
        self.failUnless(q0[0].getId() == 'a_000', 'Failed')
        return

    def testBestAnswer1(self):
        self.failUnless(self.q0.bestAnswer() == 'answer 0_2', 'Failed')
        return
    
    def testBestAnswer2(self):
        self.failUnless(self.q1.bestAnswer() == 'answer 1_2', 'Failed')
        return

#################
## Class Quizz ##
#################

class TestQuizz(TestCaseBase):
    def testCreate1(self):
        qz = Quizz(initquestions=[self.q0])
        self.failUnless(len(qz) == 1, 'Failed')
        return

    def testCreate2(self):
        qz = Quizz(initquestions=[self.q0])
        qz.addQuestion(self.q1)
        self.failUnless(len(qz) == 2, 'Failed')
        return

    def testQuestionId(self):
        qz = Quizz(initquestions=[self.q0, self.q1])
        self.failUnless(qz[0].getId() == 'q_000', 'Failed')
        return

    def testMaxiScore(self):
        qz = Quizz(initquestions=[self.q0, self.q1])
        self.failUnless(qz.maximumScore() == 4, 'Failed')
        return

    def testAnswerRender(self):
        qz = Quizz(initquestions=[self.q0, self.q1])
        gmodel = '<input type="radio" name="%s" value="%s">'
        model0 = gmodel % ('q_000', 'a_000')
        model1 = gmodel % ('q_001', 'a_000')
        self.failUnless(qz[0][0].widget() == model0, 'Failed')
        self.failUnless(qz[1][0].widget() == model1, 'Failed')
        return

    def testScore1(self):
        qz = Quizz(initquestions=[self.q0, self.q1])
        answers = {'q_000': 'a_001', 'q_001': 'a_000'}
        self.failUnless(qz.getScore(answers) == 1, 'Failed')
        return
        
    def testScore2(self):
        qz = Quizz(initquestions=[self.q0, self.q1])
        answers = {'q_000': 'a_002', 'q_001': 'a_002'}
        self.failUnless(qz.getScore(answers) == 4, 'Failed')
        return
    

#######################
## Running all tests ##
#######################

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestQuestion))
    suite.addTest(unittest.makeSuite(TestQuizz))
    return suite

if __name__ == '__main__':
    result = unittest.TextTestRunner().run(test_suite())
