#!/usr/bin/env python2.1

###########################################
## Testing features from xquizzparser.py ##
###########################################
import sys
sys.path.append('..')
import unittest
from xquizzparser import parseXQuizz
import testdata

class NormalTestCaseBase(unittest.TestCase):
    def setUp(self):
        self.errors1, self.quizz1 = parseXQuizz(testdata.quizztest1_xml)
        return

    def tearDown(self):
        pass

####################
## With valid XML ##
####################

class TestXQuizz(NormalTestCaseBase):
    def testNoError(self):
        self.failUnless(self.errors1 == [], 'Failed')
        return

    def testMaximumScore(self):
        self.failUnless(self.quizz1.maximumScore() == 7, 'Failed')
        return

    def testQuestionLabel(self):
        self.failUnless(self.quizz1[0].label == 'Do you deserve a better salary?', 'Failed')
        return

    def testAnswerLabel(self):
        self.failUnless(self.quizz1[2][2].label == 'Hmmm, what\'s a sphere?', 'Failed')
        return

    def testAnswerPoints(self):
        pts = ((0, 2), (0, 2, 1), (3, 2, 0))
        iq = -1
        for q in pts:
            iq = iq + 1
            ia = -1
            for a in q:
                ia = ia + 1
                self.failUnless(self.quizz1[iq][ia].points ==  a, 'Failed (%d, %d)' % (iq, ia))

########################
## With not valid XML ##
########################

class TestErrXQuizz(unittest.TestCase):
    def testBadRoot(self):
        errs, quizz = parseXQuizz(testdata.quizztest2_xml)
        self.failUnless(quizz == None, 'Failed, quizz should be None')
        self.failUnless(errs == ['root element should be "<quizz>"'], 'Failed, bad error message')
        return

    def testNoQuestion(self):    
        errs, quizz = parseXQuizz(testdata.quizztest3_xml)
        self.failUnless(quizz == None, 'Failed, quizz should be None')
        self.failUnless(errs == ['"<quizz>" should have one or more "<question>" elements'], 'Failed, bad error message')
        return

    def testNoLabel(self):
        errs, quizz = parseXQuizz(testdata.quizztest4_xml)
        self.failUnless(quizz == None, 'Failed, quizz should be None')
        self.failUnless(errs == ['"<question>" #1 should have one "<label>" element'], 'Failed, bad error message')
        return

    def testNotEnoughAnswers(self):
        errs, quizz = parseXQuizz(testdata.quizztest5_xml)
        self.failUnless(quizz == None, 'Failed, quizz should be None')
        self.failUnless(errs == ['"<question>" #1 should have two or more "<answer>" elements'], 'Failed, bad error message')
        return

    def testAnswerHasNoValue(self):
        errs, quizz = parseXQuizz(testdata.quizztest6_xml)
        self.failUnless(quizz == None, 'Failed, quizz should be None')
        self.failUnless(errs == ['"<answer>" #2 of "<question>" #3 should have an integer attribute named "value"'], 'Failed, bad error message')
        return

#######################
## Running all tests ##
#######################

def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestXQuizz))
    suite.addTest(unittest.makeSuite(TestErrXQuizz))
    return suite

if __name__ == '__main__':
    result = unittest.TextTestRunner().run(test_suite())
